@@ -113,9 +113,8 @@ class Service(service.RPCService, coordination.CoordinationMixin,
             CONF['service:pool_manager'].periodic_sync_retry_interval
 
         # Compute a time (seconds) by which things should have propagated
-        self.max_prop_time = (self.timeout * self.max_retries +
-                              self.max_retries * self.retry_interval +
-                              self.delay)
+        self.max_prop_time = utils.max_prop_time(self.timeout,
+            self.max_retries, self.retry_interval, self.delay)
 
     def _setup_target_backends(self):
         self.target_backends = {}
@@ -147,13 +146,15 @@ class Service(service.RPCService, coordination.CoordinationMixin,
         return topic
 
     def start(self):
-
         # Build the Pool (and related) Object from Config
         context = DesignateContext.get_admin_context()
         pool_id = CONF['service:pool_manager'].pool_id
 
         has_targets = False
 
+        # TODO(kiall): This block of code should be replaced with a cleaner,
+        #              limited version. e.g. should retry for X minutes, and
+        #              backoff rather than fixed retry intervals.
         while not has_targets:
             try:
                 self.pool = self.central_api.get_pool(context, pool_id)
@@ -171,6 +172,12 @@ class Service(service.RPCService, coordination.CoordinationMixin,
             # designate-central service may not have started yet
             except messaging.exceptions.MessagingTimeout:
                 time.sleep(0.2)
+            # designate-central failed in an unknown way, don't allow another
+            # failing / not started service to cause pool-manager to crash.
+            except Exception:
+                LOG.exception(_LE("An unknown exception occurred while "
+                                  "fetching pool details"))
+                time.sleep(5)
 
         # Create the necessary Backend instances for each target
         self._setup_target_backends()
@@ -343,6 +350,15 @@ class Service(service.RPCService, coordination.CoordinationMixin,
         timestamp_dt = datetime.fromtimestamp(timestamp)
 
         for zone in zones:
+            if isinstance(zone.created_at, datetime):
+                zone_created_at = zone.created_at
+            elif isinstance(zone.created_at, str):
+                zone_created_at = datetime.strptime(zone.created_at,
+                                                    "%Y-%m-%dT%H:%M:%S.%f")
+            else:
+                raise Exception("zone.created_at is of type %s" %
+                    str(type(zone.created_at)))
+
             if zone.status == 'DELETED':
                 # Remove any other ops for this zone
                 for zone_op in zone_ops:
@@ -350,11 +366,9 @@ class Service(service.RPCService, coordination.CoordinationMixin,
                         zone_ops.remove(zone_op)
                 # If the zone was created before the timestamp delete it,
                 # otherwise, it will just never be created
-                if (datetime.strptime(zone.created_at, "%Y-%m-%dT%H:%M:%S.%f")
-                        <= timestamp_dt):
+                if (zone_created_at <= timestamp_dt):
                     zone_ops.append((zone, 'DELETE'))
-            elif (datetime.strptime(zone.created_at, "%Y-%m-%dT%H:%M:%S.%f") >
-                  timestamp_dt):
+            elif (zone_created_at > timestamp_dt):
                 # If the zone was created after the timestamp
                 for zone_op in zone_ops:
                     if (
